# sb::Admin::Amazon
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Amazon;
use strict;
use sb::Net::Aws ();
use sb::Data::Amazon ();
use sb::Admin::Entry ();
use base qw( sb::Admin::Entry );
use vars qw( $VERSION );
$VERSION = '0.12';
# 0.12 [2009/02/09] modified title_name to make it clearer
# 0.11 [2008/11/15] modified _display_existed_items due to change of sb::Data::Amazon
# 0.10 [2007/11/05] added title_name
# 0.09 [2007/09/30] modified _display_search_result to handle empty value correctly
# 0.08 [2007/09/24] changed _update_new_item to handle asin
# 0.07 [2007/09/16] ported from Serene Bach 3.*
# 0.06 [2006/07/22] changed _display_amazon_list to use new sb::Aws
# 0.05 [2006/09/30] changed _update_amazon to check permission
# 0.04 [2005/07/26] changed _update_amazon to add new ordering function
# 0.03 [2005/07/25] changed _display_amazon_list to use sb::Aws->mode_table
# 0.02 [2005/07/20] changed _is_editable to fix a bug
# 0.01 [2005/06/30] changed _display_image to add alt and title attribute to img tag
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines
sub TEMPLATE        (){ 'amazon.html' };
sub LIST_COLUMN     (){ 4 };
sub MYLIST_COLUMN   (){ 7 };
sub DISPLAY_NUMBER  (){ 10 };
sub ITEM_LENGTH     (){ 40 };
sub NO_IMAGE        (){ '&nbsp;' };
sub DENIED_CHECK    (){ '-' };
sub DEFAULT_COLUMN  (){ '-' };
sub SUFFIX_AUTHOR   (){ ' ...' };
# --------------------------------------
# // override base class
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	return sb->string(' &gt; Favorites');
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg = undef;
	if ($self->regi() eq 'add')
	{ # add new recommended item
		$msg = $self->_update_new_item();
	}
	elsif ($self->cgi->value('update') eq '' and $self->cgi->value('submit') ne '')
	{ # update info of selected items
		$msg = $self->_update_selected_items();
	}
	else
	{ # update item info
		$msg = $self->_update_order();
	}
	return $self->SUPER::_display('newtext'=>$msg) if ($self->mode eq 'new');
	$self->build_list();
	return $self->_display('message'=>$msg);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		@_
	);
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	if ($self->cgi->value('amazon_word') ne '')
	{ # search results
		$self->_display_search_result($temp);
	}
	else
	{ # display existed items
		$self->_display_existed_items($temp);
	}
	$temp->num(0);
	$temp->tag('sb_amazon_word'=>sb::Text->new($self->cgi->value('amazon_word'))->entitized());
	my $selector  = '';
	my @aws_table = sb::Net::Aws->get_genre($self->conf->value('basic_aws_locale'));
	foreach my $genre (@aws_table)
	{
		$selector .= '<option value="' . $genre . '"';
		$selector .= ' selected="selected"' if ($genre eq $self->cgi->value('amazon_genre'));
		$selector .= '>' . sb->string($genre) . '</option>';
	}
	$temp->tag('sb_amazon_genre'=>$selector);
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - update
sub _update_new_item
{
	my $self = shift;
	my $creator = $self->cgi->value('aws_creator');
	$creator =~ s/&sb;/\n/g; # remove special escape chars
	my $url = $self->cgi->value('aws_url');
	$url = $1 if ($url =~ /(.*?)\?.+$/);
	my %elem = (
		'pid'     => ( $self->user ) ? $self->user->id : 0,
		'creator' => sb::Text->new($creator)->entitized(),
		'stat'    => 1,
		'url'     => $url,
		'date'    => $self->time->data(),
		'tz'      => $self->conf->value('conf_timezone'),
	);
	foreach my $key ('name','asin','group','release','maker','offer_price')
	{
		my $value = $self->cgi->value('aws_' . $key);
		$elem{$key} = sb::Text->new($value)->entitized();
	}
	my $new = sb::Data->add('Amazon',%elem);
	return sb->string('Unknown error.') if (!$new);
	foreach my $img ('image_small','image_medium','image_large')
	{
		my $url = $self->cgi->value('aws_' . $img . '_url');
		next if ($url eq '');
		$new->$img({
			'url'    => sb::Text->new($url)->entitized(),
			'width'  => int($self->cgi->value('aws_' . $img . '_width')),
			'height' => int($self->cgi->value('aws_' . $img . '_height')),
		});
	}
	sb::Data->update($new);
	return sb->string('Your request has been processed.');
}
sub _update_selected_items
{
	my $self = shift;
	my @sels = split("\0",$self->cgi->value('sel'));
	my @amazon = sb::Data->load('Amazon','cond'=>{'id'=>\@sels});
	ACTION_SWITCH: {
		$_ = $self->cgi->value('regi_action');
		/^entry$/ && do
		{ # create article with selected items
			my $newtext = '';
			foreach my $item (@amazon)
			{
				$newtext .= $item->formated_item;
			}
			$self->mode('new');
			return $newtext;
		};
		/^del$/ && do
		{ # delete selected items
			foreach my $item (@amazon)
			{
				$item->erase;
			}
			last ACTION_SWITCH;
		};
		/^stat(\d)$/ && do
		{ # change status
			my $new = $1;
			foreach my $item (@amazon)
			{
				$item->stat($new);
			}
			last ACTION_SWITCH;
		};
		/^order$/ && do
		{ # change order
			my $target = $amazon[0];
			@amazon = $self->change_order(
				'data'      => [ sb::Data->load('Amazon','sort'=>'order','order'=>1) ],
				'target'    => $target,
				'direction' => 0,
			);
			last ACTION_SWITCH;
		};
	} # end of ACTION_SWITCH
	sb::Data->update(@amazon) if (@amazon);
	return ($self->cgi->value('regi_action') eq 'del')
		? sprintf(sb->string('Deleted %d item(s).<br />'),$#amazon + 1)
		: sb->string('Modified.<br />');
}
sub _update_order
{
	my $self = shift;
	my @ids = split("\0",$self->cgi->value('sel_id'));
	my @amazon = sb::Data->load('Amazon','cond'=>{'id'=>\@ids});
	my $order = undef;
	foreach my $item (@amazon)
	{
		my $text = $self->cgi->value('aws_message' . $item->id);
		if ($self->cgi->value('update') ne '' and $self->has_permission('id'=>$item->pid))
		{
			$item->message(sb::Text->new($text)->entitized());
		}
		$order = $item if ($self->cgi->value('dn' . $item->id) ne '');
		$order = $item if ($self->cgi->value('up' . $item->id) ne '');
	}
	if ($order)
	{ # change the order
		@amazon = $self->change_order(
			'data'      => [ sb::Data->load('Amazon','sort'=>'order','order'=>1) ],
			'target'    => $order,
			'direction' => ($self->cgi->value('up' . $order->id) ne '') ? +1 : -1,
		);
	}
	sb::Data->update(@amazon) if (@amazon);
	return sb->string('Modified.<br />');
}
# --------------------------------------
# // private functions - display
sub _display_search_result
{
	my $self = shift;
	my $temp = shift;
	my $aws = sb::Net::Aws->new;
	my $genre = $self->cgi->value('amazon_genre');
	my $word = $self->cgi->value('amazon_word');
	my $page = ($self->cgi->value('page') eq '') ? 1 : int($self->cgi->value('page'));
	$page = int($self->cgi->value('now_page')) + 1 if ( $self->cgi->value('next') ne '' );
	$page = int($self->cgi->value('now_page')) - 1 if ( $self->cgi->value('prev') ne '' );
	my $result = $aws->get_data(
		'genre'    => ($genre eq 'ASIN') ? undef : $genre,
		'keyword'  => ($genre eq 'ASIN') ? undef : $word,
		'asin'     => ($genre eq 'ASIN') ? $word : undef,
		'page'     => $page,
		'locale'   => $self->conf->value('basic_aws_locale'),
		'id'       => $self->user->aws,
		'charcode' => $self->lang->charcode,
	);
	if ($result)
	{
		my %table = sb::Data::Amazon->table_for_aws();
		for (my $i=0;$i<$aws->count;$i++)
		{
			my $item = $result->[$i];
			$temp->num($i);
			foreach my $key (keys %table)
			{
				my $tag = 'sb_aws_' . $table{$key};
				my $value = ($item->{$key} eq '') ? '' : $item->{$key};
				if ($key eq 'creators')
				{
					$temp->tag($tag . '_one'=>''); # initialize
					if (ref($item->{$key}) eq 'ARRAY')
					{
						my @creators = @{$item->{$key}};
						my $firstone = ($#creators > 0) ? $creators[0] . SUFFIX_AUTHOR : $creators[0];
						$value = join('&sb;',@creators);
						$temp->tag($tag . '_one' => sb::Text->new($firstone)->entitized());
					}
				}
				elsif ($key =~ /image_/)
				{
					if (ref($item->{$key}) eq 'HASH')
					{
						$value = $item->{$key}->{'url'};
						$temp->tag($tag . '_width' => int($item->{$key}->{'width'}));
						$temp->tag($tag . '_height' => int($item->{$key}->{'height'}));
					}
					$tag .= '_url';
				}
				$temp->tag($tag => sb::Text->new($value)->entitized());
				if ($key eq 'image_small')
				{
					$temp->tag('sb_aws_image' => ($value ne '')
						? sb::Text->new($value)->entitized()
						: sb::Data::Amazon->noimage()
					);
				}
			}
			$temp->tag('sb_list_class'=>($i % 2) ? 'odd' : 'even');
		}
		$temp->block('sb_amazon_result'=>$aws->count);
	}
	else
	{
		$self->set_message($temp,$aws->error);
	}
	$temp->num(0);
	$temp->tag('page_now'=>$page);
	$temp->tag('sb_list_page'=>$self->display_pagelink(
		'mode'    => 'amazon',
		'column'  => LIST_COLUMN,
		'all'     => $aws->matched,
		'printed' => $aws->count,
		'num'     => DISPLAY_NUMBER,
		'start'   => 1,
		'end'     => $aws->page,
		'params'  => ['amazon_genre','amazon_word'],
	));
	$temp->block('sb_amazon_search_head'=>1);
	$temp->block('sb_amazon_search_lead'=>1);
}
sub _display_existed_items
{
	my $self = shift;
	my $temp = shift;
	my $page = $self->get_current_page();
	my @amazon = sb::Data->load('Amazon',
		'sort'  => 'order',
		'id'    => ( $self->cgi->value('aid') eq '' ) ? undef : $self->cgi->value('aid'),
		'num'   => DISPLAY_NUMBER,
		'bgn'   => $page * DISPLAY_NUMBER,
		'order' => 1,
	);
	my $matched = sb::Data->matched;
	$self->listmain(
		'template' => $temp,
		'block'    => 'sb_amazon_list',
		'objects'  => \@amazon,
		'tags'     => {
			'sb_aws_id'       => 'id',
			'sb_aws_image'    => \&_display_image,
			'sb_aws_name'     => \&_display_linked_name,
			'sb_aws_auth'     => \&_display_author,
			'sb_aws_msg'      => 'message',
			'sb_aws_date'     => 'date',
			'sb_aws_dispstat' => \&_display_status,
			'sb_aws_stat'     => 'stat',
			'sb_aws_sel'      => \&_display_checkbox,
			'sb_aws_edit'     => \&_is_editable,
		},
	);
	if ($self->cgi->value('aid') ne '' and $amazon[0])
	{
		$temp->num(0);
		$temp->tag('sb_aws_one_id'          => $amazon[0]->id);
		$temp->tag('sb_aws_one_name'        => $amazon[0]->name);
		$temp->tag('sb_aws_one_release'     => $amazon[0]->release);
		$temp->tag('sb_aws_one_maker'       => $amazon[0]->maker);
		$temp->tag('sb_aws_one_creator'     => sb::Text->new($amazon[0]->creator)->format('form'=>1));
		$temp->tag('sb_aws_one_offer_price' => $amazon[0]->offer_price);
		$temp->tag('sb_aws_one_image'       => $amazon[0]->image_as_tag('type'=>'available'));
		$temp->tag('sb_aws_one_message'     => sb::Text->new($amazon[0]->message)->format('form'=>1));
		$temp->tag('sb_aws_order'           => '-</td><td>-');
		$temp->tag('sb_list_selall_status'  => 'disabled="disabled"');
	}
	else
	{
		$temp->num(0);
		$temp->tag('page_now'=>$page);
		$self->dispnum_selector(
			'cms'  => $temp,
			'now'  => DISPLAY_NUMBER,
		);
		$temp->tag('sb_list_page'=>$self->display_pagelink(
			'mode'    => 'amazon',
			'column'  => MYLIST_COLUMN,
			'all'     => $matched,
			'printed' => $#amazon + 1,
			'num'     => DISPLAY_NUMBER,
		));
		$self->order_marker(
			'template' => $temp,
			'objects'  => \@amazon,
			'matched'  => $matched,
			'tag'      => 'sb_aws_order',
			'page'     => $page,
			'number'   => DISPLAY_NUMBER,
		);
	}
	$temp->block('sb_amazon_mylist_head'=>1);
	$temp->block('sb_amazon_mylist_lead'=>1);
	$temp->block('sb_amazon_mylist'     =>1);
	$temp->block('sb_amazon_select_list'=>($self->cgi->value('aid') eq '') ? 1 : 0);
	$temp->block('sb_amazon_select'     =>($self->cgi->value('aid') eq '') ? 1 : 0);
	$temp->block('sb_amazon_one'        =>($self->cgi->value('aid') eq '') ? 0 : 1);
	$temp->block('sb_amazon_one_only'   =>($self->cgi->value('aid') eq '') ? 0 : 1);
	$temp->block('sb_amazon_one_status' =>($self->cgi->value('aid') eq '') ? 0 : 1);
}
# --------------------------------------
# // private functions - list handlers
sub _display_image
{
	my $self = shift;
	my $obj  = shift;
	return $self->linked_text(
		'text' => $obj->image_as_tag(),
		'base' => '?__mode=amazon&amp;aid=' . $obj->id,
		'user' => $obj->pid,
	);
}
sub _display_linked_name
{
	my $self = shift;
	my $obj  = shift;
	return $self->linked_text(
		'text' => $obj->name,
		'base' => $obj->url,
	);
}
sub _is_editable
{
	my $self = shift;
	my $obj  = shift;
	return ($self->has_permission('id'=>$obj->pid)) 
		? '' 
		: 'disabled="disabled"';
}
sub _display_author
{
	my $self = shift;
	my $obj  = shift;
	my $pid  = ( defined($self->users->{$obj->pid}) ) ? $obj->pid : 0;
	return( $self->users->{$pid}->real );
}
sub _display_status
{
	my $self = shift;
	my $obj  = shift;
	return $self->list_status(
		'stat'   => $obj->stat,
		'string' => sb->string('close:open'),
	);
}
sub _display_checkbox
{
	my $self = shift;
	my $obj  = shift;
	return ( $self->has_permission('id'=>$obj->pid) and $self->cgi->value('aid') eq '' )
		? '<input type="checkbox" name="sel" value="' . $obj->id . '" />'
		: DENIED_CHECK;
}
1;
__END__
